/*
 * SketchyLISP -- An interpreter for purely applicative Scheme
 * Copyright (C) 2005,2006,2007 Nils M Holm <nmh@t3x.org>
 * See http://www.t3x.org/sketchy/license.html for conditions of use.
 */

#ifdef __SKETCHYLIB__
 #define extern
#endif

#define SK_MAJOR	31
#define SK_RELEASE	"2007-02-18"

/*
 * Number of nodes and vector cells.
 * Memory = Nodes * (2 * sizeof(int) + 1) 
 *          + Vcells * sizeof(int)
 * (Recommended: 16-bit: 12280, >=32-bit: 131072)
 */
#ifdef SMALL
#define	SK_DFL_NODES	12280
#else
#define	SK_DFL_NODES	131072
#endif

#define SK_DFL_VCELLS	SK_DFL_NODES

/*
 * Neither the node pool nor the vector pool
 * should be smaller than this.
 */
#define SK_MIN_SIZE	12280

/* Maximum length for symbols */
#ifdef SMALL
#define	SK_TEXTLEN	128
#else
#define	SK_TEXTLEN	1024
#endif

/* Max. path length */
#define SK_MAXPATHL	256

/* Default image */
#ifndef SK_DFL_IMAGE
 #define SK_DFL_IMAGE	"/usr/local/share/sketchy/sketchy.image"
#endif

/* For errors without specific source. */
#define SK_NOEXPR	-1

/* End of input indicator */
#define SK_EOT		-1

/* Maximum number of user-addd primitives */
#define SK_MAX_USER_PRIMITIVES	128

enum {
	SK_TYPE_BOOLEAN,
	SK_TYPE_CHAR,
	SK_TYPE_EOF,
	SK_TYPE_INTEGER,
	SK_TYPE_PAIR,
	SK_TYPE_PROCEDURE,
	SK_TYPE_STRING,
	SK_TYPE_SYMBOL,
	SK_TYPE_SYNTAX,
	SK_TYPE_UNDEFINED,
	SK_TYPE_VOID
};

/* Counter structure used for statistics. */
struct sk_counter {
	int	n,	/* ones */
		n1k,	/* thousands */
		n1m,	/* millions */
		n1g;	/* billions */
};

/* Structure for GC statistics. */
struct sk_gcStats {
	int	nodes_used,
		vcells_used,
		nodes_max,
		vcells_max;
};

/* Shared variables. */
extern int	SK_errFlag;	/* Error flag */
extern int	SK_statFlag;	/* Statistics flag */
extern int	SK_strictApply;	/* Strict (R5RS) apply flag */
extern int	SK_closureForm;	/* Ext. rep. of closures (0,1,2) */
extern int	SK_arrowMode;	/* Enable arrow comments */
extern int	SK_metaChar;	/* Meta command character */
extern int	SK_trace;	/* Function being traced */
extern char	*SK_errMsg;	/* Most recent error message */
extern char	SK_errArg[SK_TEXTLEN];	/* Additional information */
extern int	SK_errExpr;	/* Expression causing last error */
extern char	SK_errFile[SK_MAXPATHL];	/* File of last error */
extern int	SK_errLine;	/* Line number of last error */
extern int	SK_errFun;	/* Function of last error */
extern int	SK_errFrame;	/* Call frame of last error */
extern struct sk_counter
		SK_reductions,	/* Reduction counter */
		SK_allocations,	/* Allocation counter */
		SK_collections;	/* Garbage collection counter */

/* Trace Handler; NULL = print trace */
extern int (*SK_traceHandler)(int n);

/*
 * Function prototypes
 */
int	sk_addUserPrim(char *name, int (*handler)(int n));
int	sk_args(int n);
void	sk_bye(void);
int	sk_car(int n);
int	sk_cdr(int n);
int	sk_char(int n);
int	sk_cons(int car, int cdr);
void	sk_count(struct sk_counter *c, int k);
void	sk_display(int n);
void	sk_dumpImage(char *p);
void	sk_dumpSymbols(char *p);
int	sk_eof(void);
int	sk_error(char *msg, int n);
int	sk_eval(int n);
const int	sk_false(void);
char	*sk_findSource(char *p, char *buf);
int	sk_findSym(char *s);
void	sk_fini(void);
void	sk_gc(struct sk_gcStats *stats);
void	sk_gotError(void);
int	sk_init(int nodes, int vcells);
long	sk_int(int n, int *ep);
char	**sk_license(void);
int	sk_listToVector(int n);
int	sk_load(char *p);
int	sk_loadImage(char *p);
int	sk_mkChar(int c);
int	sk_mkInteger(long v);
int	sk_mkString(char *s, int k);
const int	sk_nil(void);
void	sk_nl(void);
int	sk_nthArg(int n, int i);
void	sk_pr(char *s);
void	sk_print(int n);
void	sk_printCallTrace(int n);
void	sk_printCounter(struct sk_counter *c);
void	sk_printError(void);
void	sk_printStats(void);
void	sk_prnum(int n, int w);
void	sk_protect(int n);
int	sk_rdch(void);
int	sk_read(void);
int	sk_readFromString(char *s);
void	sk_reject(int c);
int	sk_require(char *p);
void	sk_resetCounter(struct sk_counter *c);
void	sk_setCar(int n, int new);
void	sk_setCdr(int n, int new);
void	sk_stop(void);
const char	*sk_string(int n);
const int	sk_true(void);
int	sk_typeof(int n);
const int	sk_undefined(void);
void	sk_unprotect(int k);
const int	*sk_vector(int n);
int	sk_vector_len(int n);
const int	sk_void(void);
int	sk_writeToString(int n);
