; SketchyLISP Extension Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; filter

; ---conformance---
; R5.91RS

; ---purpose---
; Extract members that match a given predicate from a list.

; ---args---
; P - predicate
; A - list

; ---keywords---
; FILTER function, lists, filtering

; ---see-also---
; memp, remp

; ---example---
; (filter number? '(abc 123 def 456 ghi)) => (123 456)

; (require "nullp.scm") ; null?
; (require "lib/reverse.scm")

; ---model---
; (define (filter p a)
;   (cond ((null? a) '())
;     ((p (car a))
;       (cons (car a) (filter p (cdr a))))
;     (else (filter p (cdr a)))))

; ---code---
(define (filter p a)
  (letrec
    ((_filter
       (lambda (a b)
         (cond ((null? a) b)
           ((p (car a))
             (_filter (cdr a) (cons (car a) b)))
           (else (_filter (cdr a) b))))))
    (_filter (reverse a) '())))
