; SketchyLISP Extension Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; product

; ---conformance---
; SketchyLISP Extension

; ---purpose---
; Compute the product of a sequence of
; positive numbers.

; ---args---
; N - least value of sequence
; M - maximum value of seqence

; ---keywords---
; PRODUCT function, product of sequence

; ---see-also---
; sum, iota

; ---example---
; (product 5 7) => 210

; (require "lib/digits.scm")
; (require "lib/equal.scm") ; =
; (require "lib/greater.scm") ; >
; (require "lib/times.scm") ; *
; (require "lib/natural.scm")

; ---model---
; (define (product n m)
;   (cond ((= n m) m)
;     (else (* n (product (+ n 1) m)))))

; ---code---
(define (product n m)
  (let ((nn (natural n))
       (lim (natural m)))
    (letrec
      ((prod
         (lambda (n r)
           (cond ((n> n lim) r)
             (else (prod (+ n 1) (* n r)))))))
      (cond ((n> nn lim)
          (bottom '(product: bad range)))
        (else (prod nn 1))))))
