; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; assoc

; ---conformance---
; R5RS

; ---purpose---
; Retrieve an association from an association list.
; An association list is a list of pairs where
; the
; .E car
; part of each pair holds a key and the
; .E cdr
; part of the pair holds the value
; associated with that key:
; .B
; .C ((key1 . value1) ... (keyN . valueN))
; .B
; .C Assoc
; returns the first association whose key is equal
; to a given datum.

; ---args---
; X - key of value to be found
; A - association list

; ---keywords---
; ASSOC function, association lists, alists, associate

; ---see-also---
; assv, assq, member

; ---example---
; (assoc '3 '((1 . i) (2 . ii) (3 . iii) (4 . iv))) => (3 . iii)

(define assoc #t)

; (require "nullp.scm") ; null?
(require "equalp.scm") ; equal?
(require "caar.scm") ; cdar

; ---code---
(define (assoc x a)
  (cond ((null? a) #f)
    ((equal? (caar a) x) (car a))
    (else (assoc x (cdr a)))))
