; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; <

; ---conformance---
; R5RS

; ---purpose---
; Check whether a seqeunce of numbers is in strict ascending
; order. Return
; .C #t,
; if
; .V a<b<...
; and otherwise
; .C #f.

; ---args---
; A - number
; B... - numbers

; ---keywords---
; < function, less than, ordering, relation
; comparison, predicate, integer

; ---see-also---
; digits, >, <=, >=, n<

; ---example---
; (< -123 0) => #t

(define less #t)

; (require "nullp.scm") ; null?
(require "abs.scm")
(require "integer.scm")
(require "negativep.scm") ; negative?
(require "non-negativep.scm") ; non-negative?
(require "fold-left.scm")
(require "neqp.scm") ; neq?
(require "not.scm")

; ---code---
(define (< a . b)
  (letrec
    ; Check whether A has fewer members than B
    ((shorter-list
       (lambda (a b)
         (cond ((null? a)
             (not (null? b)))
           ((null? b) #f)
           (else (shorter-list (cdr a) (cdr b))))))
     (shorter
       (lambda (a b)
         (shorter-list (integer->list a)
           (integer->list b))))
     ; Check whether A has more members than B
     (longer
       (lambda (a b)
         (shorter b a)))
     ; Handle signs and lengths
     (less
       (lambda (a b)
         (cond ((negative? a)
           (cond ((non-negative? b) #t)
             ((shorter a b) #f)
             ((longer a b) #t)
             (else (n< (abs b) (abs a)))))
         ((negative? b) #f)
         ((shorter a b) #t)
         ((longer a b) #f)
         (else (n< (abs a) (abs b))))))
     (lt
       (lambda (a b)
         (cond ((eq? a #t) #t)
           ((less (integer a) (integer b)) b)
           (else #t)))))
    (cond ((null? b)
        (bottom '(too few arguments to <)))
      (else (neq? (fold-left lt a b) #t)))))
