; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; string<=?

; ---conformance---
; R5RS

; ---purpose---
; Test whether a sequence of strings is in
; lexically non-descending order.

; ---args---
; X - string
; Y... - strings

; ---keywords---
; STRING<=? function, strings, comparison, lexical order,
; non-descending

; ---see-also---
; string-ci<=?, string<?, string=?, string>?, string>=?, char<=?, equal?

; ---example---
; (string<=? "abc" "abc" "abx") => #t

(define slep #t)

; (require "cgtp.scm") ; char>?
; (require "cltp.scm") ; char<?
; (load "nullp.scm") ; null?
(require "pred-iter.scm") ; predicate-iterator

; ---code---
(define string<=?
  (letrec
    ((le?
       (lambda (x y)
         (cond ((null? x) #t)
           ((null? y) #f)
           ((char<? (car x) (car y)) #t)
           ((char>? (car x) (car y)) #f)
           (else (le? (cdr x) (cdr y)))))))
  (predicate-iterator
    (lambda (x y)
      (le? (string->list x)
           (string->list y))))))
