; SketchyLISP Example Program
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; dates

; ---purpose---
; Date computations.

; ---language---
; R5RS Scheme

; ---example---
; (date-difference '(1969 7 21) '(2000 1 1)) => 11121

; (require "lib/caar.scm") ; cadr
; (require "lib/caaar.scm") ; caaar caddr
; (require "lib/logand.scm")
; (require "lib/zerop.scm") ; zero?
; (require "lib/greater.scm") ; >
; (require "lib/remainder.scm")
; (require "lib/minus.scm") ; -
; (require "lib/plus.scm") ; +
; (require "lib/times.scm") ; *
; (require "lib/quotient.scm")
; (require "lib/list-ref.scm")
; (require "lib/less.scm") ; <

; ---code---
; Definition of the 'date' data structure.
; Dates are lists of numbers. Order of year,
; month, day is in ISO-8816 format. For instance,
; July 23rd 1968 becomes '(1968 7 23).

(define (year-of date) (car date))

(define (month-of date) (cadr date))

(define (day-of date) (caddr date))

; Convert a given date to the number of days
; since 01-01-01. This function will deliver
; wrong results for dates before 1582-10-15.

(define (date-to-days date)
  (letrec
    ((month-offsets
       #(0 31 59 90 120 151 181 212 243 273 304 334))
     (leap-year?
       (lambda (year)
         (cond ((zero? (remainder year 400)) #t)
           ((zero? (remainder year 100)) #f)
           (else (zero? (remainder year 4))))))
     (year-to-days
       (lambda (year)
         (- (+ (* year 365)
               (quotient year 4)
               (quotient year 400))
            (quotient year 100))))
     (days-so-far
       (lambda (n)
         (vector-ref month-offsets (- n 1))))
     (month-to-days
       (lambda (year month)
         (cond ((and (leap-year? year) (> month 2))
             (+ 1 (days-so-far month)))
           (else (days-so-far month))))))
    (+ (year-to-days (- (year-of date) 1))
       (month-to-days (year-of date) (month-of date))
       (day-of date))))

; Test whether two dates are in chronological order.
; Return #T, if the first date is 'before' the
; second one and otherwise #F.

(define (date-before? date1 date2)
  (cond ((< (year-of date1) (year-of date2)) #t)
        ((> (year-of date1) (year-of date2)) #f)
        ((< (month-of date1) (month-of date2)) #t)
        ((> (month-of date1) (month-of date2)) #f)
     (else (< (day-of date1) (day-of date2)))))

; Compute the difference between two dates
; in days. The range covered by the dates given
; may not include (1582 10 15) or the result will
; be wrong.

(define (date-difference date1 date2)
  (cond ((date-before? date1 date2)
      (- (date-to-days date2)
         (date-to-days date1)))
    (else (- (date-to-days date1)
             (date-to-days date2)))))

; Test wehether two dates are in reverse chronological
; order. Return #T, if the first date is 'after' the
; second one and otherwise #F.

(define (date-after? date1 date2)
  (date-before? date2 date1))
