; SketchyLISP Example Program
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; isort

; ---language---
; R5RS Scheme + SketchyLISP Ext.

; ---purpose---
; Sort a list using insertion sort. A predicate is
; used to define the desired order of members. It
; should return
; .C #t
; for each two members in the desired order.

; ---args---
; A - list to sort
; P - predicate

; ---keywords---
; ISORT function, sorting, ordering, insertion sort

; ---example---
; (isort < '(5 1 3 2 4)) => (1 2 3 4 5)

; ---model---
; (define isort (lambda (p a)
;   (cond ((null? a) '())
;     (else (insert p (car a) (isort p (cdr a)))))))

; ---code---
(require "insert.scm")

(define (isort p a)
  (letrec
    ((_isort
       (lambda (a r)
         (cond ((null? a) r)
           (else (_isort (cdr a)
                   (insert p (car a) r)))))))
    (_isort a '())))
